from pretty_bad_protocol import gnupg
from pathlib import Path
import shutil


def import_pubkey(pubkey):
    """This function takes as argument the public key filepath, reads the key from file and adds it to the keychain.
    The key will be used in encrypt_info to encrypt the file.
    It returns a gpg object that contains all the info of the keychain (that has only that one public key)."""

    gpg = gnupg.GPG(homedir='./keys')

    try:
        if Path(pubkey).exists():
            try:
                with open(pubkey, 'rb') as key_file:
                    data = key_file.read()
                    gpg.import_keys(data)
            except OSError as e:
                return f"Can't open the public key file: {e}"

        elif not Path(pubkey).is_file():
            return "The path to the public key file doesn't appear to be valid"
        else:
            return "An unknown error occurred, while trying to read the public key from file."

    # This is a design decision that hasn't been implemented, where you can give the public key as text instead of file.
    # This exception would be raised if pubkey was not a path but instead a long string,
    # in which case Path(pubkey).exists raises an OSError.
    except OSError:
        try:
            gpg.import_keys(pubkey)
        except Exception as e:
            return f"This doesn't seem to be a valid public key: {e}"

    return gpg


def encrypt_info(gpg, info_file):
    """This function takes as arguments the gpg keychain and the filepath to the file to be encrypted
    It saves the file to disk and returns its path, which is ./file/<filename_as_it_was>.gpg"""

    if Path(info_file).exists() and Path(info_file).is_file():
        key_id = gpg.list_keys()[0]['keyid']  # The keychain contains only one key (index=0), read the keyid of that key.

        name = Path(info_file).name
        suffix = Path(info_file).suffix
        encrypted_path = Path('files/')
        if not encrypted_path.exists():
            Path(encrypted_path).mkdir()

        # Append the suffix .gpg to the file's filename. That way the filename of the original file is kept.
        encrypted_file = encrypted_path.joinpath(Path(f'{name}').with_suffix(suffix + '.gpg'))

        try:
            with open(info_file, 'rb') as message:
                try:
                    with open(encrypted_file, 'wb') as encrypted:
                        try:
                            gpg.encrypt(message, key_id, output=encrypted)
                            # Remove the public key from hard disk
                            shutil.rmtree('./keys')
                        except Exception as e:
                            shutil.rmtree('./keys')
                            return f"Fatal error. The file couldn't be encrypted. Error: {e}"
                except OSError as e:
                    shutil.rmtree('./keys')
                    return f"Fatal error. The target file couldn't be opened. Error: {e}"
        except OSError as e:
            shutil.rmtree('./keys')
            return f"Fatal error. The source file couldn't be opened. Error: {e}"

        return encrypted_file  # This is a Path object

    else:
        # Remove the public key from hard disk
        shutil.rmtree('./keys')

        return f"Fatal error. The source file couldn't be found."


if __name__ == '__main__':
    print(
        "If you want to run this script standalone, edit it and provide the necessary parameters to the function calls")
    gpg = import_pubkey('')  # Path to public key

    if gpg is not None:
        print(encrypt_info(gpg, ''))  # Path to file to encrypt

    # Remove the public key from hard disk
    shutil.rmtree('./keys')



import smtplib
from email.message import EmailMessage
from pathlib import Path


def send_email(sender, receiver_address, sender_address, password, content_file=None,
               subject=None, unencrypted_message=None):
    """This function is responsible for sending the emails, both warning and final"""
    email = EmailMessage()
    email['from'] = sender
    email['to'] = receiver_address
    email['subject'] = subject

    if unencrypted_message is not None:
        email.set_content(unencrypted_message, 'text')

    if content_file is not None:
        email.make_mixed()

        try:
            with open(content_file, 'rb') as attachment:
                email.add_attachment(attachment.read(), maintype='application', subtype='octate-stream', filename=Path(content_file).name)
        except OSError as e:
            return f"Fatal error. Unable to open or attach the encrypted file: {e}"

    try:
        with smtplib.SMTP(host='smtp.gmail.com', port=587) as smtp:
            smtp.ehlo()
            smtp.starttls()
            smtp.login(sender_address, password)
            smtp.send_message(email)
            return 0
    except smtplib.SMTPException as e:
        return f"Sending email failed: {e}"


if __name__ == '__main__':
    print("If you want to run this script standalone, edit it and provide the necessary parameters to the function call")


from crontab import CronTab
import os
from pathlib import Path
from argparse import ArgumentParser


def cron_job():
    my_cron = CronTab(user=True)

    parser = ArgumentParser(description="Set cron job parameters")
    parser.add_argument('-s', '--shell', help="Set the shell environment. Default is sh.")
    parser.add_argument('-p', '--path', help="Set a custom PATH for cron. Default is \"/usr/bin:/bin\"")
    parser.add_argument('-m', '--mailto',
                        help="Set a local to whom emails are sent by the cron job. "
                             "Set to \"\" to deactivate emails altogether. Default is current user.")
    parser.add_argument('-t', '--time', default='12:00',
                        help="Set the time when the cron job will be run. Default is 12:00")
    parser.add_argument('-d', '--display', action='store_true', help="Display the active configuration and cron jobs")
    parser.add_argument('-r', '--remove', action='store_true', help="Delete the cron job")

    args = parser.parse_args()

    if args.shell:
        my_cron.env['SHELL'] = args.shell
    if args.path:
        my_cron.env['PATH'] = args.path
    if args.mailto:
        my_cron.env['MAILTO'] = args.mailto
    if args.display:
        os.system('crontab -l')
    elif args.remove:
        my_cron.remove_all(comment='check in')
        my_cron.write()
        print("** Cron job deleted")
    else:
        my_cron.remove_all(comment='check in')
        job = my_cron.new(command=f'cd {Path.cwd()} && python3 check_in.py', comment='check in')
        job.day.every(1)
        job.hour.on(args.time.split(':')[0].strip())
        job.minute.on(args.time.split(':')[1].strip())

        my_cron.write()
        os.system('crontab -l')


if __name__ == '__main__':
    cron_job()




from tkinter import *
from tkinter import filedialog
from tkinter import ttk
from pathlib import Path
import json
from datetime import date
import file_encryption


class App:
    def __init__(self, root):
        """This class creates the layout of the GUI with all the entry fields and labels.
        It also stores the values in two json files: parameters.json and auth.json.
        parameters.json contains all the parameters in the left column that concern the email with the private info
        auth.json contains all the parameters necessary to send emails and SMS (including credentials)
        If the details have already been filled previously, it populates the entry fields with them
        Finally, it calls the file_encryption functions to encrypt the file and save it to disk"""

        root.title('Last Will')

        # Create one big frame for the two smaller frames that are arranged in two columns
        big_frame = ttk.Frame(root)
        big_frame.pack()

        left_frame = ttk.Frame(big_frame)
        right_frame = ttk.Frame(big_frame)
        left_frame.grid(row=0, column=0, sticky=N)
        right_frame.grid(row=0, column=1, sticky=N)

        # Two lists with the labels and the entries' keys for the left entry fields
        self.left_labels = ["Name to appear as sender", "Recipient's email", "Email subject", "Unencrypted message",
                            "File to encrypt", "Recipient's PUBLIC Key"]
        self.left_entry_labels = ['sender_name', 'recipient_email', 'subject', 'unencrypted_message',
                                  'file_to_encrypt', 'pubkey_file']

        # Calls labels_entries that creates the labels and entries for the left column
        self.left_entries = self.labels_entries(left_frame, self.left_labels, self.left_entry_labels)
        self.entry_values = {}

        # Creates two buttons that call the respective functions that bring up a browse window
        ttk.Button(left_frame, text='Browse', command=self.file_to_encrypt_dialog).grid(row=4, column=2, sticky='W')
        ttk.Button(left_frame, text='Browse', command=self.pubkey_dialog).grid(row=5, column=2, sticky='W')

        # Two lists with the labels and the entries' keys for the right entry fields
        self.right_labels = ["Email to send from", "Email's password", "Your email", "Your phone number",
                             "Your Twilio phone number",
                             "Your Twilio account ID", "Your Twilio Auth token"]
        self.right_entry_labels = ['email', 'email_pwd_secret', 'your_email', 'your_phone', 'twilio_phone',
                                   'account_sid_secret', 'auth_token_secret']

        # Again calls labels_entries that creates the labels and entries for the right column
        self.right_entries = self.labels_entries(right_frame, self.right_labels, self.right_entry_labels)

        # Retrieve the values if they exist already and fills out the entry fields
        self.retrieve_values()

        # This creates an empty Label frame where messages will be displayed
        self.message_frame = ttk.Label(big_frame, text="")
        self.message_frame.grid(row=1, columnspan=2)

        # This creates a frame at the bottom for the buttons and the buttons Quit and Update
        button_frame = ttk.Frame(big_frame)
        button_frame.grid(row=2, columnspan=2)

        ttk.Button(button_frame, text='Quit', command=root.quit).pack(side=LEFT)  # Quits the app
        # Updates parameters.json and auth.json with the values in the entry fields. Also, encrypts the file.
        ttk.Button(button_frame, text='Update', command=self.update).pack(side=LEFT)

    def labels_entries(self, frame, labels, entry_labels):
        """This method creates the labels and entry fields for the GUI based on the lists provided as arguments"""

        entries = {}
        for index, label in enumerate(labels):
            # The Text frame requires different initialization than the Entry frames
            if entry_labels[index] != 'unencrypted_message':
                ttk.Label(frame, text=label).grid(row=index, sticky='E')

                # If the key contains the word 'secret' it means it's a credential, so show asterisks in the field
                if 'secret' in entry_labels[index]:
                    entries[entry_labels[index]] = ttk.Entry(frame, width=35, show='*')
                else:
                    entries[entry_labels[index]] = ttk.Entry(frame, width=35)
            else:
                ttk.Label(frame, text=label).grid(row=index, sticky='NE')
                entries[entry_labels[index]] = Text(frame, height=10, width=40, borderwidth=1, relief=SUNKEN)

            entries[entry_labels[index]].grid(row=index, column=1, sticky='W')

        return entries

    def file_to_encrypt_dialog(self):
        """This method opens a browse window and receives the path of the file to be encrypted.
        Then it adds the path to the entry field"""

        filename = filedialog.askopenfilename(initialdir='', title="Select a file")

        # If the entry field isn't empty, delete the entry first and then replace it with the new filename
        if filename != '':
            self.left_entries['file_to_encrypt'].delete(0, END)
        self.left_entries['file_to_encrypt'].insert(0, filename)

    def pubkey_dialog(self):
        """This method opens a browse window and receives the path of the public key file.
        Then it adds the path to the entry field"""

        filename = filedialog.askopenfilename(initialdir='', title="Select a file")

        # If the entry field isn't empty, delete the entry first and then replace it with the new filename
        if filename != '':
            self.left_entries['pubkey_file'].delete(0, END)
        self.left_entries['pubkey_file'].insert(0, filename)

    def update(self):
        """This method is called when the Update button is clicked.
        It reads the entries from the entry fields and updates parameters.json and auth.json.
        Any empty fields are stored as "", which guarantees that the keys will exist.
        It also calls file_encryption functions to encrypt the file and then saves it"""

        # Get the values from the left entry fields
        for key, value in self.left_entries.items():
            if key == 'unencrypted_message':
                self.entry_values[key] = value.get(1.0, END)
            else:
                self.entry_values[key] = value.get()

        # Get the values from the parameters.json file.
        # We want to compare the filenames in the file with the ones in the fields, to check if they've changed.
        try:
            with open('files/parameters.json', 'r') as params:
                entry_values_in_file = json.loads(params.read())
        except OSError as e:
            self.message_frame.configure(text=f"Fatal error. Couldn't open parameters.json. Error: {e}")
        except TypeError as e:
            self.message_frame.configure(text=f"Couldn't write Parameter values as JSON. Error: {e}")

        # Call import_pubkey from file_enryption and get back the keychain
        gpg = file_encryption.import_pubkey(self.entry_values['pubkey_file'])

        # If the return value is a string, then an error occurred, which is displayed in self.message_frame
        # Also we'll keep the filenames already in parameters.json
        successfully_encrypted = False
        if type(gpg) is not str:
            # Call encrypt_info from file encryption and get back the path to the encrypted file (a Path object)
            encrypted_file = file_encryption.encrypt_info(gpg, self.entry_values['file_to_encrypt'])

            # If the return value is a string or None, then an error occurred, which is displayed in self.message_frame
            # Also we'll keep the filenames already in parameters.json
            if type(encrypted_file) is not str:
                self.message_frame.configure(text=f"File encrypted successfully: {Path(encrypted_file).name}")
                successfully_encrypted = True
            else:
                self.message_frame.configure(text=encrypted_file + "\nThe encrypted and pubkey files haven't changed.")
        else:
            self.message_frame.configure(text=gpg + "\nThe encrypted and pubkey files haven't changed.")

        # We keep the filenames already in parameters.json
        if not successfully_encrypted:
            self.entry_values['file_to_encrypt'] = entry_values_in_file['file_to_encrypt']
            self.entry_values['pubkey_file'] = entry_values_in_file['pubkey_file']

        # Delete any pre-existing .gpg files in the ./files folder
        existing_file = Path('files').joinpath(Path(self.entry_values['file_to_encrypt']).name + '.gpg')
        for file in Path('files').iterdir():
            if file.suffix == '.gpg' and file != existing_file:
                Path(file).unlink()

        # Write the values in parameters.json
        try:
            with open('files/parameters.json', 'w') as params:
                params.write(json.dumps(self.entry_values, ensure_ascii=False))
        except OSError as e:
            self.message_frame.configure(text=f"Fatal error. Couldn't open parameters.json. Error: {e}")
        except TypeError as e:
            self.message_frame.configure(text=f"Couldn't write Parameter values as JSON. Error: {e}")

        # Read the values of the right entry fields and write them to auth.json
        try:
            with open('files/auth.json', 'w') as auth:
                auth_dict = {key: value.get() for key, value in self.right_entries.items()}
                auth.write(json.dumps(auth_dict))
        except OSError as e:
            self.message_frame.configure(text=f"Couldn't open parameters.json. Error: {e}")
        except TypeError as e:
            self.message_frame.configure(text=f"Couldn't write Auth values as JSON. Error: {e}")

        print(self.message_frame.grab_current())

    def retrieve_values(self):
        """This method retrieves the values from the JSON files and populates the entry fields."""

        if Path('files/parameters.json').exists():
            try:
                with open('files/parameters.json', 'r') as params:
                    entry_values = json.loads(params.read())
                for key, value in entry_values.items():
                    if key != 'unencrypted_message':
                        self.left_entries[key].insert(0, value)
                    else:
                        self.left_entries[key].insert(1.0, value)
            except OSError as e:
                self.message_frame.configure(text=f"Couldn't open parameters.json to read the entries. "
                                                  f"If this the first time your run the app ignore this error. "
                                                  f"Error: {e}")

        if Path('files/auth.json').exists():
            try:
                with open('files/auth.json', 'r') as auth:
                    auth_dict = json.loads(auth.read())

                for key in self.right_entry_labels:
                    try:
                        self.right_entries[key].insert(0, auth_dict[key])
                    except Exception as e:
                        self.right_entries[key].insert(0, "")
            except OSError as e:
                self.message_frame.configure(text=f"Couldn't open auth.json to read the entries. "
                                                  f"If this the first time your run the app ignore this error. "
                                                  f"Error: {e}")


if __name__ == '__main__':
    root = Tk()  # Create the window
    root.geometry("+300+300")  # Place it in the middle of the screen
    app = App(root)

    if not Path('files/').exists():
        Path('files').mkdir()

    # When you open the app this is considered a check in and the current date is written to check_in.json
    # This means that if there's a deadline key in the file it's removed
    try:
        with open('files/check_in.json', 'w') as check_in:
            check_in.write(json.dumps({'last_check_in': str(date.today())}))
    except OSError as e:
        print(f"Fatal error. Couldn't open check_in.json to write the current date. Error: {e}")

    root.mainloop()
    # root.attributes('-topmost', True)
    # root.attributes('-topmost', False)



from emails import send_email
from twilio.rest import Client
from pathlib import Path
from datetime import date, datetime
import json
from crontab import CronTab


class Check:
    def __init__(self, last_check_in, deadline=None, warning_sent=False):
        self.last_check_in = last_check_in  # The last day last_will.py was run
        self.deadline = deadline  # The deadline datetime, if more than 30 have passed
        self.warning_sent = warning_sent  # True if a warning about deadline has already been sent

        try:
            with open('files/parameters.json') as params_file:
                parameters = json.loads(params_file.read())
        except OSError as e:
            print(f"Fatal error. Couldn't open parameters.json. Error: {e}")
            exit(1)

        self.sender_name = parameters['sender_name']
        self.recipient_email = parameters['recipient_email']
        self.subject = parameters['subject']
        self.unencrypted_message = parameters['unencrypted_message']

        try:
            with open('files/auth.json', 'r') as auth_file:
                auth = json.loads(auth_file.read())
        except OSError as e:
            print(f"Fatal error. Couldn't open auth.json. Error: {e}")
            exit(1)

        self.email = auth['email']
        self.email_pwd = auth['email_pwd_secret']
        self.your_email = auth['your_email']
        self.your_phone = auth['your_phone']
        self.twilio_phone = auth['twilio_phone']
        self.account_sid = auth['account_sid_secret']
        self.auth_token = auth['auth_token_secret']

    def dead(self):
        """Function to send the email to the designated recipient if you don't check in after the deadline has passed"""

        encrypted_file = ''
        for file in Path('files').iterdir():
            if file.suffix == '.gpg':
                encrypted_file = str(file)

        if encrypted_file == '':
            print("Fatal error. Didn't find an encrypted file in folder './files/")
            exit(1)

        send_email(self.sender_name, self.recipient_email,
                   self.email, self.email_pwd,
                   encrypted_file, self.subject, self.unencrypted_message)

        print("\nEmail with important info sent. Rest in peace, your money is in good hands\n")

        # Remove the cron job. No point in checking any longer.
        my_cron = CronTab(user=True)
        my_cron.remove_all(comment='check in')

        exit(0)

    def send_warning(self):
        """Function to send warnings to email and optionally as SMS,
        when at least 30 days have passed since the last check in"""

        # Check whether all the necessary parameters for SMS are present
        if self.your_phone != '' and self.twilio_phone != '' and self.account_sid != '' and self.auth_token != '':
            client = Client(self.account_sid, self.auth_token)

            try:
                sms = client.messages.create(
                    body="""Last will: It was at least 30 days since your last check in. 
                            This is a reminder to check in in the next 24 hours.""",
                    from_=self.twilio_phone,
                    to=self.your_phone)
                sms
                print("\nSMS sent")
            except Exception as e:
                print(f"An error occurred while trying to send the SMS. Error: {e}")

        else:
            print("\nMissing SMS parameters. SMS not sent")

        # Check whether all the necessary parameters for email are present
        if self.sender_name != '' and self.recipient_email != '' and self.email != '' and self.email_pwd != '':
            message = f"""It has been at least 30 days since you last checked in. 
You need to check in in the next 24 hours.\n
Otherwise at {self.deadline} the email with the important info will be sent to the designated recipient.\n
In order to reset simply go to the working directory and run python3 last_will.py"""

            # send_email will return 0 if everything went ok, otherwise it will return an error message
            status = send_email(self.sender_name, self.your_email,
                                self.email, self.email_pwd,
                                subject='Last will: Reminder to check in', unencrypted_message=message)

            if status != 0:
                print(status)
                exit(1)
            else:
                print("Email sent\n")

                print(f"You have until {self.deadline} to check in. "
                      f"In order to do that simply go to the working directory and run ./last_will.sh\n")
        else:
            print("Missing email parameters. Email not sent.\n")
            exit(1)


if __name__ == '__main__':
    if Path('files/check_in.json').exists():
        try:
            with open('files/check_in.json', 'r') as check_in_file:
                check_in_info = json.loads(check_in_file.read())
        except OSError as e:
            print(f"Fatal error. Couldn't open check_in.json. Error: {e}")
            exit(1)

        try:
            last_check_in = datetime.strptime(check_in_info['last_check_in'], "%Y-%m-%d").date()
        except ValueError as e:
            print(f"Fatal error. The last check in date is not in the right format. Error: {e}")
            exit(1)

        days_since_check_in = (date.today() - last_check_in).days

        checker = Check(last_check_in)

        if days_since_check_in > 30:
            # If there's no deadline key in check_in.json create it and put the deadline 24 hours from now.
            if 'deadline' not in check_in_info.keys():
                deadline = datetime.now().replace(day=date.today().day + 1, microsecond=0)
                check_in_info['deadline'] = str(deadline)

                try:
                    with open('files/check_in.json', 'w') as check_in_file:
                        check_in_file.write((json.dumps(check_in_info)))
                except OSError as e:
                    print(f"Fatal error. Couldn't open check_in.json to write the deadline. Error: {e}")

            # If a deadline has already been set, read it from the file and set the warning_sent flag to true
            # so that no more warnings are sent.
            else:
                deadline = datetime.strptime(check_in_info['deadline'], "%Y-%m-%d %H:%M:%S")
                checker.warning_sent = True

            checker.deadline = deadline

            if deadline < datetime.now():  # If the deadline has been passed, send the private info email
                checker.dead()
            elif not checker.warning_sent:  # Else if not passed and no warning has been sent, send the warning
                checker.send_warning()
            else:
                print("Warning already sent")

        # If last check in was less than 30 days ago, simply let the user know.
        else:
            print(f"You checked in {days_since_check_in} days ago. You have {30 - days_since_check_in} "
                  f"days more before you'll need to check in again.")

    else:
        print("There is no check_in.json file. Please run ./last_will.sh and fill out the relevant info")



from setuptools import setup
from os import path

this_directory = path.abspath(path.dirname(__file__))
with open(path.join(this_directory, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()

setup(
    name='last_will',
    packages=['last_will'],
    version='0.1.9',
    license='MIT',
    description='A program to send encrypted sensitive information to a trusted person in case of your untimely demise',
    long_description=long_description,
    long_description_content_type='text/markdown',
    author='Michalis Fragkiadakis',
    author_email='michalis.fr@icloud.com',
    url='https://github.com/michalisFr/last_will',
    download_url='https://github.com/michalisFr/last_will/archive/v0.1.9.tar.gz',
    keywords=['encrypted', 'sensitive information', 'failsafe'],
    install_requires=[
        'certifi',
        'chardet',
        'idna',
        'pretty-bad-protocol',
        'psutil',
        'PyJWT',
        'python-crontab',
        'python-dateutil',
        'pytz',
        'requests',
        'six',
        'twilio',
        'urllib3'
    ],
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: End Users/Desktop',
        'Topic :: Utilities',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
    ],
)
